# -*- coding: utf-8 -*-

import logging
from queue import Queue, Empty

from buoy.base.data.item import Status
from buoy.base.database import DeviceDB
from buoy.base.device.threads.base import BaseThread

logger = logging.getLogger(__name__)


class SaveThread(BaseThread):
    """
    Clase encargada de guardar los datos en la base de datos
    """

    def __init__(self, db: DeviceDB, queue_save_data: Queue, queue_notice: Queue):
        super(SaveThread, self).__init__(queue_notice)
        self.db = db
        self.queue_save_data = queue_save_data

    def activity(self):
        try:
            item = self.queue_save_data.get(timeout=self.timeout_wait)

            if item.status == Status.NEW:
                self.save(item.data)
                # TODO Habría que tener encuenta si el item se guardó antes
            elif item.status == Status.SENT:
                self.set_sent(item.data)
                # TODO Habría que tener encuenta si el item no existe
            elif item.status == Status.FAILED:
                self.set_failed(item.data)

            self.queue_save_data.task_done()
        except Empty:
            pass

    def save(self, item):
        """ Guarda el registro en la base de datos """
        logger.debug("Save to item %s" % str(item))
        self.db.save(item)

    def set_sent(self, item):
        logger.debug("Mark item with sent %s" % str(item))
        self.db.set_sent(item.uuid)

    def set_failed(self, item):
        logger.debug("Mark item with send failed %s" % str(item))
        self.db.set_failed(item.uuid)



import unittest
from queue import Queue

from nose.tools import eq_

from buoy.base.device.threads.base import BaseThread


class TestThreadBase(unittest.TestCase):
    def setUp(self):
        queue_notice = Queue()
        self.thread = BaseThread(queue_notice=queue_notice)

    def test_is_aliveReturnFalse_when_createThread(self):
        eq_(self.thread.is_alive(), False)

    def test_is_aliveReturnFalse_when_stopThread(self):
        self.thread.active = True

        self.thread.stop()
        eq_(self.thread.is_alive(), False)


if __name__ == '__main__':
    unittest.main()



import unittest
from unittest.mock import patch

from serial import SerialException

from buoy.base.device.device import Device
from buoy.base.device.exceptions import DeviceNoDetectedException


class TestDevice(unittest.TestCase):

    # TODO arreglar test
    @unittest.skip
    @patch('buoy.lib.device.device.Serial', side_effect=SerialException())
    def test_shouldReturnException_when_theDeviceIsNotPresent(self, mock_serial):
        serial_config = {
            'port': '/dev/weather_station',
            'baudrate': 4800,
            'stopbits': 1,
            'parity': 'N',
            'bytesize': 8,
            'timeout': 0
        }

        device = Device(device_name="test", db=None, serial_config=serial_config)

        self.assertRaises(DeviceNoDetectedException, device.connect)

    @patch.object(Device, 'is_open', return_value=True)
    def test_shouldReturnException_when_existsExceptionsInQueue(self, mock_is_open):
        ex = DeviceNoDetectedException(exception=Exception())
        serial_config = {
            'port': '/dev/weather_station',
            'baudrate': 4800,
            'stopbits': 1,
            'parity': 'N',
            'bytesize': 8,
            'timeout': 0
        }

        device = Device(device_name="test", db=None, serial_config=serial_config)
        device.queues['notice'].put_nowait(ex)

        self.assertRaises(DeviceNoDetectedException, device._listener_exceptions)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

import logging
from queue import Queue, Full

from buoy.base.database import DeviceDB
from buoy.base.device.threads.base import BaseThread

logger = logging.getLogger(__name__)


class DBToSendThread(BaseThread):
    """
    Clase base encargada buscar datos en la base de datos que no han sido enviados
    """

    def __init__(self, db: DeviceDB, queue_send_data: Queue, queue_notice: Queue, **kwargs):
        super(DBToSendThread, self).__init__(queue_notice,
                                             timeout_wait=kwargs.pop('timeout_wait', 300))

        self.db = db
        self.queue_send_data = queue_send_data
        self.queue_notice = queue_notice
        self.limit_queue = kwargs.pop("limit_queue", 100)

    def activity(self):
        if not self.queue_send_data.full():
            items = self.db.get_items_to_send()
            for item in items:
                try:
                    self.queue_send_data.put_nowait(item)
                except Full:
                    logger.warning("Send queue is full")
                    break



import unittest
from queue import Queue, Full
from unittest.mock import MagicMock
from unittest.mock import patch

from nose.tools import eq_
from serial import SerialException

from buoy.base.data.item import BaseItem
from buoy.base.device.exceptions import ProcessDataExecption
from buoy.base.device.threads.reader import DeviceReader

serial_config = {
    'port': '/dev/weather_station',
    'baudrate': 4800,
    'stopbits': 1,
    'parity': 'N',
    'bytesize': 8,
    'timeout': 0
}

device = MagicMock()


class DeviceReaderMock(DeviceReader):
    def __init__(self, **kwargs):
        super(DeviceReaderMock, self).__init__(**kwargs)

    def parser(self, data):
        return data


class TestItemReaderThread(unittest.TestCase):
    def setUp(self):
        self.queue_save_data = Queue()
        self.queue_send_data = Queue()
        self.queue_notice = Queue()
        self.thread = DeviceReaderMock(queue_save_data=self.queue_save_data, queue_send_data=self.queue_send_data,
                                       queue_notice=self.queue_notice, device=device)

    def test_returnTwoItems_when_passStringWith3CarriageReturnAndWhiteSpace(self):
        text = """hola
            
        adios"""

        lines = self.thread.split_by_lines(text)

        eq_(len(lines), 2)

    def test_bufferContainsJoinTwoText_when_callTwoRead_Data(self):
        text = [b"Hola", b" como esta"]

        self.thread.device.read = MagicMock(side_effect=text)

        self.thread.read_data()
        self.thread.read_data()

        eq_(self.thread.buffer, "Hola como esta")

    def test_stopThread_when_deviceRaiseException(self):
        self.thread.active = True
        self.thread.read_data = MagicMock(side_effect=OSError())

        self.thread.start()
        self.thread.join(timeout=60)

        eq_(self.thread.active, False)
        eq_(self.thread.queue_notice.qsize(), 1)

    def test_returnFalse_when_charSplitterIsInBuffer(self):
        text = """hola

        adios"""

        self.thread.buffer = text

        eq_(self.thread.is_buffer_empty(), False)

    def test_returnTrue_when_charSplitterIsNotInBuffer(self):
        text = """hola"""

        self.thread.buffer = text

        eq_(self.thread.is_buffer_empty(), True)

    @patch.object(DeviceReader, 'read_data', side_effect=SerialException())
    @patch.object(DeviceReader, 'error')
    def test_shouldStopThread_when_raiseExceptionInReadDataMethod(self, mock_read, mock_error):
        self.thread.active = True
        self.thread.activity()
        eq_(mock_error.call_count, 1)

    def test_returnArrray2ItemsAndBufferRestOfString_when_bufferHas2SplitCharAndMoreData(self):
        text = """hola
        adios
        bye"""

        self.thread.buffer = text
        self.thread.process_data()

        eq_(self.thread.queue_save_data.qsize(), 2)
        eq_(self.thread.buffer, "bye")

    def test_returnArrray2ItemsAndBufferEmpty_when_bufferHas2SplitCharOnly(self):
        text = """hola
        adios
"""

        self.thread.buffer = text
        self.thread.process_data()

        eq_(self.thread.queue_save_data.qsize(), 2)
        eq_(len(self.thread.buffer), 0)

    def test_returnException_when_bufferHasNotSplitChar(self):
        text = """hola"""
        self.thread.buffer = text

        self.assertRaises(ProcessDataExecption, self.thread.process_data)

    def test_raiseException_when_queuesIsFull(self):
        self.queue_save_data.put_nowait = MagicMock(side_effect=Full())
        self.queue_send_data.put_nowait = MagicMock(side_effect=Full())
        item = BaseItem()

        self.thread.put_in_queues(item)

        eq_(self.queue_save_data.qsize(), 0)
        eq_(self.queue_send_data.qsize(), 0)


if __name__ == '__main__':
    unittest.main()



from unittest.mock import MagicMock


class SerialMock(object):
    def __init__(self, **kwargs):
        self.read = MagicMock(return_value=b'Hola')
        self.in_waiting = MagicMock(return_value=4)
        self.write = MagicMock()
        self.close = MagicMock()
        self.is_open = MagicMock(return_value=True)



import unittest

from buoy.tests.item_db_to_send_thread import DBToSendThreadTest
from buoy.tests.item import Item


class TestItemInDBToSendThread(DBToSendThreadTest):
    __test__ = True
    item_cls = Item


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

import logging
from os import path

import yaml

logger = logging.getLogger(__name__)


def load_config(path_config):
    if not path.isfile(path_config):
        logger.error("No exists config file %s" % (path_config,))

    f = open(path_config, 'r')
    config = yaml.load(f)
    f.close()

    return config


def load_config_device(device_name, path_config='/etc/buoy/buoy.yaml'):
    config = load_config(path_config=path_config)

    return config['device'][device_name]


def load_config_device_serial(device_name, path_config='/etc/buoy/device.yaml'):
    config = load_config_device(device_name, path_config=path_config)

    return config['device'][device_name]['serial']


def load_config_logger(path_config='/etc/buoy/logging.yaml'):
    config = load_config(path_config=path_config)

    return config



# -*- coding: utf-8 -*-

import logging
from copy import copy
from queue import Queue, Full
from typing import List

from serial import Serial

from buoy.base.data.item import ItemQueue, BaseItem
from buoy.base.device.exceptions import LostConnectionException, ProcessDataExecption
from buoy.base.device.threads.base import DeviceBaseThread
from buoy.base.data.item import BufferItems

logger = logging.getLogger(__name__)


class DeviceReader(DeviceBaseThread):
    """ Clase encargada de leer y parsear los datos que devuelve el dispositivo """

    def __init__(self, device: Serial, queue_notice: Queue, **kwargs):
        self.char_splitter = kwargs.pop('char_splitter', '\n')
        self.interval = kwargs.pop("interval", None)

        super(DeviceReader, self).__init__(device, queue_notice)

        self.buffer = ''
        self.buffer_items = BufferItems(interval=self.interval)

        self.queue_save_data = kwargs.pop('queue_save_data', None)
        if not self.queue_save_data:
            logging.info("No save data in database")
        self.queue_send_data = kwargs.pop('queue_send_data', None)
        if not self.queue_send_data:
            logging.info("No send data in real-time")

    def activity(self):
        try:
            self.read_data()
            logger.debug("Waiting data")
            if not self.is_buffer_empty():
                self.process_data()

        except (OSError, Exception) as ex:
            logger.error("Device disconnected")
            self.error(LostConnectionException(exception=ex))

    def read_data(self):
        logger.debug("Data in buffer %s" % self.buffer)
        self.buffer += self.device.read(self.device.in_waiting).decode()

    def is_buffer_empty(self):
        return self.char_splitter not in self.buffer

    def process_data(self):
        logger.debug("Proccessing data: %s", self.buffer)
        buffer = self.buffer.rsplit(self.char_splitter, 1)
        try:
            self.buffer = buffer[1].strip()
        except IndexError as ex:
            raise ProcessDataExecption(message="Proccesing data without char split", exception=ex)
        for line in self.split_by_lines(buffer[0]):
            item = self.parser(line)
            if item:
                aggs_item = self.buffer_items.append(item)
                if aggs_item:
                    self.put_in_queues(aggs_item)
            logger.debug("Received data - " + line)

    def split_by_lines(self, buffer: str) -> List[str]:
        lines = buffer.split(self.char_splitter)
        return [l.strip() for l in lines if len(l.strip())]

    def parser(self, data) -> BaseItem:
        pass

    def put_in_queues(self, item: BaseItem):
        logger.debug("Item readed from device - %s" % str(item))

        if self.queue_save_data and not self.queue_save_data.full():
            try:
                self.queue_save_data.put_nowait(ItemQueue(data=copy(item)))
            except Full:
                logger.error("Save data queue is full")

        if self.queue_send_data and not self.queue_send_data.full():
            try:
                self.queue_send_data.put_nowait(item)
            except Full:
                logger.warning("Send data queue is full")



# -*- coding: utf-8 -*-

import logging
import signal
import sys
import time
from os import getpid, makedirs, remove, EX_OK, EX_OSERR
from os.path import isfile, exists, join

logger = logging.getLogger(__name__)


def get_config(device_name, buoy_config):
    serial_config = buoy_config['device'][device_name]['serial']
    mqtt_config = buoy_config['device'][device_name]['mqtt']
    db_config = buoy_config['database']
    service_config = buoy_config['service']

    return serial_config, mqtt_config, db_config, service_config


class DaemonException(Exception):
    pass


class PID(object):
    """
        Clase para la gestión del PID de un servicio utilizando un fichero
    """

    def __init__(self, **kwargs):
        daemon_config = kwargs.pop('daemon_config')
        self.daemon_name = kwargs.pop('daemon_name')
        self.path_pidfile = daemon_config['path_pidfile']
        self.pid = getpid()
        self.pid_file = join(self.path_pidfile, self.daemon_name + ".pid")
        self.create_path_pid_file()

    def create_path_pid_file(self):
        if not exists(self.path_pidfile):
            makedirs(self.path_pidfile)

    def create_pid_file(self):
        if isfile(self.pid_file):
            remove(self.pid_file)

        with open(self.pid_file, 'w') as f:
            f.write(str(self.pid))

    def remove_pid_file(self):
        if isfile(self.pid_file):
            remove(self.pid_file)


class Daemon(PID):
    """
        Clase base para la creación de un servicio linux
        Cuenta con un ciclo de vida:
            * before_start
            * start
            * run
            * before_stop
            * stop
    """

    def __init__(self, **kwargs) -> None:
        self.start_timeout = kwargs.pop('start_timeout', 0)
        super(Daemon, self).__init__(**kwargs)

        self._active = False

        signal.signal(signal.SIGINT, self.handler_signal)
        signal.signal(signal.SIGTERM, self.handler_signal)

    def handler_signal(self, signum, frame):
        """ Maneja la captura de señales de interrupción, poniendo el servicio en modo inactivo """
        self._active = False

    def _before_start(self):
        self._active = True
        self.create_pid_file()
        self.before_start()

    def before_start(self):
        """ Función que se ejecuta antes de iniciar el servicio """
        pass

    def is_active(self):
        return self._active

    def start(self):
        logger.info("Start service")

        self._before_start()
        time.sleep(self.start_timeout)
        try:
            self.run()
        except Exception as ex:
            self.error()

        self._stop()

    def run(self):
        """ Función donde implementar la lógica del servicio """
        pass

    def _before_stop(self):
        self.before_stop()

    def before_stop(self):
        """ Función que se ejecuta antes de parar el servicio """
        pass

    def _stop(self, code=EX_OK):
        if self.is_active():
            self._active = False
            self._before_stop()
            self.remove_pid_file()
            sys.exit(code)

    def stop(self):
        logger.info("Stop service")
        self._stop()

    def error(self):
        """ Función que se ejecuta cuando se produce un error """
        logger.error("Service exit status fail")
        self._stop(code=EX_OSERR)



from queue import Queue
from unittest.mock import MagicMock

from nose.tools import ok_

from buoy.base.data.item import Status
from buoy.base.database import DeviceDB
from buoy.base.device.threads.mqtt import MqttThread, MQTT_ERR_SUCCESS
from buoy.tests.base_device_tests import *
from buoy.tests.item import *


class FakeDeviceDB(DeviceDB):
    def __init__(self):
        pass


class FakeReponseMQTT(object):
    def __init__(self, rc=0, mid=1):
        self.rc = rc
        self.mid = mid

    def wait_for_publish(self):
        pass


class FakeMQTT(object):
    def __init__(self):
        pass

    def disconnect(self):
        pass


class TestMqttThread(unittest.TestCase):
    def setUp(self):
        self.cls = Item
        self.topic = "redmic/data"
        self.qos = 1
        self.thread = MqttThread(db=FakeDeviceDB(), queue_send_data=Queue(), queue_data_sent=Queue(),
                                 queue_notice=Queue())

    def fill_limbo(self, size):
        items = get_items(size)
        for idx, item in enumerate(items):
            self.thread.limbo.add(idx, item)

    @patch.object(MqttThread, 'is_connected_to_mqtt', return_value=False)
    @patch.object(MqttThread, 'send')
    def test_onceCallToConnectMqtt_when_inititializeThread(self, mock_send, mock_is_connected_to_mqtt):
        self.thread.activity()

        eq_(mock_is_connected_to_mqtt.call_count, 1)
        eq_(mock_send.call_count, 0)

    def test_deleteItemInLimbo_when_itemSuccessPublished(self):
        self.fill_limbo(size=2)

        self.thread.on_publish(None, None, 1)

        eq_(self.thread.limbo.size(), 1)
        ok_(self.thread.limbo.get(1) is None)

    def test_isConnectIsTrue_when_onConnectCallbackReceiveRCEqualZero(self):
        self.fill_limbo(size=2)

        self.thread.on_connect(None, None, flags={"session present": 1}, rc=0)

        ok_(self.thread.is_connected_to_mqtt())

    def test_isConnectIsFalse_when_onConnectCallbackReceiveRcDistinctZero(self):
        self.fill_limbo(size=2)

        self.thread.on_connect(None, None, None, rc=1)

        ok_(self.thread.is_connected_to_mqtt() is False)

    def test_clearLimboAndStopThread_when_disconnectMqttOk(self):
        self.fill_limbo(size=2)
        self.thread.active = True
        client = FakeMQTT()
        client.loop_stop = MagicMock()

        self.thread.on_disconnect(client, None, 0)

        ok_(self.thread.is_connected_to_mqtt() is False)
        ok_(self.thread.is_active() is False)
        eq_(self.thread.limbo.size(), 0)

    def test_clearLimboAndDontStopThread_when_disconnectMqttKO(self):
        self.fill_limbo(size=2)
        self.thread.active = True
        client = FakeMQTT()

        self.thread.on_disconnect(client, None, 1)

        ok_(self.thread.is_connected_to_mqtt() is False)
        ok_(self.thread.is_active() is True)
        eq_(self.thread.limbo.size(), 0)

    def test_itemInsideLimbo_when_sendItem(self):
        item_expected = get_item()
        mid = item_expected.uuid
        client = FakeMQTT()
        client.publish = MagicMock(return_value=FakeReponseMQTT(rc=MQTT_ERR_SUCCESS, mid=mid))
        self.thread.client = client

        self.thread.send(item_expected)

        eq_(self.thread.limbo.size(), 1)
        eq_(self.thread.limbo.get(mid), item_expected)

    def test_limboIsEmpty_when_failedSentItem(self):
        item_expected = get_item()
        client = FakeMQTT()
        client.publish = MagicMock(side_effect=ValueError('Error sent item'))
        self.thread.client = client

        self.thread.send(item_expected)

        eq_(self.thread.queue_data_sent.qsize(), 1)
        item = self.thread.queue_data_sent.get_nowait()
        eq_(item_expected, item.data)
        ok_(item.status == Status.FAILED)
        eq_(self.thread.limbo.size(), 0)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

import logging
from typing import List, AnyStr

import psycopg2
from psycopg2 import DatabaseError, IntegrityError, errorcodes
from psycopg2.extensions import AsIs
from psycopg2.extras import DictCursor, DictRow, register_uuid

from buoy.base.data.item import BaseItem

register_uuid()
logger = logging.getLogger(__name__)


class DeviceDB(object):
    """ Clase encargada de gestionar la base de datos """

    def __init__(self, db_config, db_tablename, cls_item, **kwargs):

        self.connection = None
        self.connect(db_config)
        self.tablename_data = db_tablename
        self.cls = cls_item
        self.window_time = kwargs.pop('window_time', 300)

        self._insert_sql = """INSERT INTO """ + self.tablename_data + """(%s) VALUES %s RETURNING uuid"""
        self._find_by_id_sql = """SELECT * FROM """ + self.tablename_data + """ WHERE uuid = ANY(%s)"""
        self._update_status_sql = """UPDATE """ + self.tablename_data + """ SET sent=%s WHERE uuid = ANY(%s)"""
        self._select_items_to_send_sql = """SELECT * FROM """ + self.tablename_data + \
                                         """ WHERE sent IS false AND num_attempts < %s """ + \
                                         """AND date < now() - %s * interval '1 second' """ + \
                                         """ORDER BY date LIMIT %s"""

    def connect(self, db_config):
        logger.debug("Connecting to database")
        self.connection = psycopg2.connect(**db_config)

    def save(self, item: BaseItem) -> BaseItem:
        """ Inserta un nuevo registro en la base de datos """
        try:
            with self.get_cursor() as cur:
                sql = self.create_insert_sql(item, cur)
                cur.execute(sql)
                item.id = cur.fetchone()[0]
            self.connection.commit()
        except IntegrityError as e:
            if e.pgcode == errorcodes.UNIQUE_VIOLATION:
                logger.warning("The data exists in database yet")
            else:
                logger.exception("No insert data", exc_info=e)
        except DatabaseError as e:
            logger.exception("No insert data", exc_info=e)

        return item

    def get(self, identifier):
        """ Retorna un registro un registro dado un identificador """
        with self.get_cursor() as cur:
            sql = cur.mogrify(self._find_by_id_sql, (identifier,))
            cur.execute(sql)
            row = cur.fetchone()

        return row

    def _get_items_to_send(self, *args):
        """ Retorna la lista de registros nuevos a enviar """
        with self.get_cursor() as cur:
            sql = cur.mogrify(self._select_items_to_send_sql, *args)
            cur.execute(sql)
            rows = cur.fetchall()

        items = []
        for row in rows:
            items.append(self.cls(**row))

        return items

    def get_items_to_send(self, **kwargs) -> List[DictRow]:

        window_time = kwargs.pop('window_time', 900)
        max_attemps = kwargs.pop('max_attemps', 3)
        size = kwargs.pop('size', 100)

        return self._get_items_to_send((max_attemps, window_time, size))

    def update_status(self, uuids: List, status=True):
        if len(uuids):
            try:
                with self.get_cursor() as cur:
                    sql = cur.mogrify(self._update_status_sql, (status, uuids))
                    cur.execute(sql)
                self.connection.commit()
            except DatabaseError:
                logger.exception("No update data")

    def set_sent(self, uuid):
        self.update_status([uuid], status=True)

    def set_failed(self, uuid):
        self.update_status([uuid], status=False)

    def create_insert_sql(self, item, cursor):
        columns = self.__get_column_names(item)
        values = [getattr(item, column) for column in columns]
        sql = cursor.mogrify(self._insert_sql, (AsIs(','.join(columns)), tuple(values)))

        return sql

    def get_cursor(self):
        return self.connection.cursor(cursor_factory=psycopg2.extras.DictCursor)

    @staticmethod
    def __get_column_names(item: BaseItem) -> List[AnyStr]:
        """ Retorna una lista con el nombre de las columnas
        :param item: BaseItem
        :return: list
        """
        columns = list(dict(item).keys())

        return columns



import unittest
from datetime import datetime, timezone
from os import path

from nose.tools import eq_, ok_

from buoy.base.database import DeviceDB
from buoy.tests.database import *
from buoy.tests.item import Item


class BaseDBTests(unittest.TestCase):
    item_class = None
    db_tablename = None
    db_cls = DeviceDB
    skip_test = False
    path_sql = 'tests/support/data'

    @classmethod
    def setUpClass(cls):
        global skip_test

        if cls is BaseDBTests:
            skip_test = True
        else:
            skip_test = False

        super(BaseDBTests, cls).setUpClass()

    def setUp(self):
        """ Module level set-up called once before any tests in this file are executed. Creates a temporary database
        and sets it up """

        if skip_test:
            self.skipTest("Skip BaseTest tests, it's a base class")

        global db_conf

        db_conf = prepare_db(path.join(self.path_sql, 'setup.sql'))

    def tearDown(self):
        """ Called after all of the tests in this file have been executed to close the database connecton and destroy
        the temporary database """

        close_db()

    def test_add_item_in_db(self):

        item_to_insert = self.item_class(**self.data)

        dev_db = self.db_cls(
            db_config=db_conf,
            db_tablename=self.db_tablename,
            cls_item=self.item_class
        )

        item = dev_db.save(item_to_insert)

        rows = apply_sql_clause("""SELECT * FROM %s""" % (self.db_tablename,))

        eq_(len(rows), 1)
        row = rows[0]

        if not hasattr(self, "data_expected"):
            self.data_expected = self.data

        for key, value in self.data_expected.items():
            eq_(row[key], value)

        eq_(row['uuid'], item.uuid)

    def test_update_status_items_in_db(self):

        dev_db = self.db_cls(
            db_config=db_conf,
            db_tablename=self.db_tablename,
            cls_item=self.item_class
        )

        sql_clause = """SELECT * FROM %s ORDER BY date""" % (self.db_tablename,)
        before_rows = apply_sql_clause(sql_clause)

        uuids = list()
        for row in before_rows:
            uuids.append(row['uuid'])
        dev_db.update_status(uuids)

        after_rows = apply_sql_clause(sql_clause)

        for idx, aft_row in enumerate(after_rows):
            eq_(aft_row['sent'], True)
            ok_(aft_row['uuid'] == before_rows[idx]['uuid'])
            ok_(aft_row['num_attempts'] > before_rows[idx]['num_attempts'])

    def test_should_return15Items_when_getItemsToSend(self):
        apply_sql_file(path.join(self.path_sql, 'data_example.sql'))
        dev_db = self.db_cls(
            db_config=db_conf,
            db_tablename=self.db_tablename,
            cls_item=self.item_class
        )
        rows = dev_db.get_items_to_send()

        eq_(len(rows), 15)
        ok_(all(a.date <= b.date for a, b in zip(rows[:-1], rows[1:])))

    def test_should_returnZeroItems_when_getItemsToSend(self):
        apply_sql_file(path.join(self.path_sql, 'data_not_send.sql'))

        dev_db = self.db_cls(
            db_config=db_conf,
            db_tablename=self.db_tablename,
            cls_item=self.item_class
        )

        rows = dev_db.get_items_to_send()

        eq_(len(rows), 0)


class TestDeviceDB(BaseDBTests):
    item_class = Item
    db_tablename = "device"
    data = {
        'date': datetime.now(tz=timezone.utc),
        'value': 30.3273
    }


if __name__ == '__main__':
    unittest.main()



import unittest
from datetime import datetime, timezone
from buoy.tests.item_save_thread import ItemSaveThreadTest
from buoy.tests.item import Item


def get_item():
    data = {
        'date': datetime.now(tz=timezone.utc).strftime("%Y-%m-%dT%H:%M:%S.%f%z"),
        'value': '26.8'
    }
    return Item(**data)


class TestItemSaveThread(ItemSaveThreadTest):
    __test__ = True


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

import argparse
from os import path


def is_valid_file(filename):
    if not path.isfile(filename):
        raise argparse.ArgumentTypeError("{0} does not exist".format(filename))

    return filename


def parse_args(**kwargs):
    path_config = kwargs.pop('path_config', '/etc/buoy/')
    file_config = kwargs.pop('file_config', 'device.yaml')
    parser = argparse.ArgumentParser()
    parser.add_argument("--config-file", help="Ruta al fichero de configuración del servicio",
                        default=path.join(path_config, file_config), type=is_valid_file)
    parser.add_argument("--config-log-file", help="Ruta al fichero de configuración de los logs",
                        default=path.join(path_config, 'logging.yaml'), type=is_valid_file)
    return parser.parse_args()



# -*- coding: utf-8 -*-

import logging
from queue import Queue, Empty

from serial import Serial, SerialException

from buoy.base.device.threads.mqtt import MqttThread
from buoy.base.device.threads.resender import DBToSendThread
from buoy.base.device.threads.save import SaveThread
from buoy.base.device.exceptions import DeviceNoDetectedException

logger = logging.getLogger(__name__)


class Device(object):
    def __init__(self, *args, **kwargs):
        self.serial_conf = kwargs.pop('serial', None)
        self.db = kwargs.pop('db')

        self.cls_reader = kwargs.pop('cls_reader', None)
        self.reader_conf = kwargs.pop('reader', {})
        self.cls_writer = kwargs.pop('cls_writer', None)
        self.cls_save = kwargs.pop('cls_save', SaveThread)
        self.cls_send = kwargs.pop('cls_send', MqttThread)
        self.cls_reader_from_db = kwargs.pop('cls_reader_from_db', DBToSendThread)
        self.mqtt_conf = kwargs.pop('mqtt', None)

        self.qsize_send_data = kwargs.pop('qsize_send_data', 1000)

        self.queues = {}
        self._create_queues()

        # Device
        self.name = kwargs.pop('device_name')
        self._dev_connection = None

    def _create_queues(self):
        for queue_name in ['notice', 'write_data', 'save_data', 'send_data']:
            qsize = 0
            if queue_name == 'send_data':
                qsize = self.qsize_send_data
            self.queues[queue_name] = Queue(maxsize=qsize)

    def run(self):
        try:
            self.connect()
            self._create_threads()
            self._start_threads()
            self.configure()
            self._listener_exceptions()
        except Exception as ex:
            logger.error(ex)
            raise ex

    def connect(self):
        logger.info("Connecting to device")
        try:
            self._dev_connection = Serial(**self.serial_conf)
        except SerialException as ex:
            raise DeviceNoDetectedException(process=self.name, exception=ex)
        logger.info("Connected to device")

    def _create_threads(self):
        if self.cls_writer:
            self._thread_writer = self.cls_writer(device=self._dev_connection,
                                                  queue_write_data=self.queues['write_data'],
                                                  queue_notice=self.queues['notice'])
        if self.cls_reader:
            self._thread_reader = self.cls_reader(device=self._dev_connection,
                                                  queue_save_data=self.queues['save_data'],
                                                  queue_send_data=self.queues['send_data'],
                                                  queue_notice=self.queues['notice'],
                                                  **self.reader_conf)
        if self.cls_save:
            self._thread_save = self.cls_save(queue_save_data=self.queues['save_data'],
                                              queue_notice=self.queues['notice'],
                                              db=self.db)
        if self.cls_reader_from_db:
            self._thread_send = self.cls_reader_from_db(queue_send_data=self.queues['send_data'],
                                                        queue_notice=self.queues['notice'],
                                                        db=self.db)
        if self.cls_send:
            self._thread_send = self.cls_send(queue_send_data=self.queues['send_data'],
                                              queue_data_sent=self.queues['save_data'],
                                              queue_notice=self.queues['notice'],
                                              **self.mqtt_conf)

    def _start_threads(self):
        self._run_action_threads(action='start')

    def _run_action_threads(self, action='start'):
        prefix = '_thread_'
        names = ['reader', 'writer', 'save', 'send', 'reader_from_db']

        for name in names:
            field = prefix + name
            if hasattr(self, field):
                thread = getattr(self, field)
                getattr(thread, action)()

    def configure(self):
        pass

    def _listener_exceptions(self):
        while self.is_open():
            try:
                ex = self.queues['notice'].get(timeout=0.2)
                raise ex
            except Empty:
                pass

    def disconnect(self):
        logger.info("Disconnecting to device")
        self._stop_threads()
        if self.is_open():
            self._dev_connection.close()
        logger.info("Disconnected to device")

    def _stop_threads(self):
        self._run_action_threads(action='stop')

    def is_open(self):
        return self._dev_connection and self._dev_connection.is_open() and self.is_active()

    def write(self, data):
        self.queues['write_data'].put_nowait(data + "\r")



# -*- coding: utf-8 -*-

import logging
import time
from queue import Queue
from threading import Thread

from serial import Serial

logger = logging.getLogger(__name__)


class BaseThread(Thread):
    def __init__(self, queue_notice: Queue, **kwargs):
        self.timeout_wait = kwargs.pop('timeout_wait', 0.2)
        super(BaseThread, self).__init__(**kwargs)
        self.active = False
        self.queue_notice = queue_notice

    def run(self):
        logging.info("Start thread %s", self.__class__.__name__)
        self.before_activity()
        self.active = True
        while self.is_active():
            self.activity()
            time.sleep(self.timeout_wait)
        self.after_activity()

    def is_active(self) -> bool:
        """
        Retorna el estado del hilo, activo o parado

        :return: Estado del hilo
        """
        return self.active

    def before_activity(self):
        pass

    def after_activity(self):
        pass

    def activity(self):
        """
        Función donde implementar el proceso a ejecutar el hilo
        :return:
        """
        pass

    def stop(self):
        """ Para el hilo """
        self.active = False
        logging.info("Stop thread %s", self.__class__.__name__)

    def error(self, exception):
        self.queue_notice.put_nowait(exception)
        self.stop()


class DeviceBaseThread(BaseThread):
    def __init__(self, device: Serial, queue_notice: Queue, **kwargs):
        super(DeviceBaseThread, self).__init__(queue_notice)
        self.device = device

    def is_active(self):
        return super().is_active() and self.device.is_open



from datetime import datetime, timezone

from buoy.base.data.item import BaseItem


class Item(BaseItem):
    def __init__(self, **kwargs):
        self.value = kwargs.pop('value', None)
        super(Item, self).__init__(**kwargs)

    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, value):
        self._value = self._convert_string_to_decimal(value)


def get_item(**kwargs):
    date = kwargs.pop("date", datetime.now(tz=timezone.utc).strftime("%Y-%m-%dT%H:%M:%S.%f%z"))
    data = {
        'date': date,
        'value': '26.8'
    }
    return Item(**data)


def get_items(num=2, dates=None):
    items = []
    for i in range(0, num):
        if dates:
            item = get_item(date=dates[i])
        else:
            item = get_item()
        items.append(item)
    return items



import unittest

from nose.tools import eq_, ok_

from buoy.base.device.threads.mqtt import Limbo
from buoy.tests.item import *


class TestLimbo(unittest.TestCase):

    def test_itemAdded_when_callAddItemLimbo(self):
        limbo = Limbo()
        item = get_item()
        limbo.add(1, item)

        ok_(limbo.exists(1))
        eq_(limbo.size(), 1)

    def test_clearLimbo_when_addedItemsAndClearedLimbo(self):
        limbo = Limbo()

        items = get_items(2)
        for idx, item in enumerate(items):
            limbo.add(idx, item)

        eq_(limbo.size(), 2)
        limbo.clear()
        eq_(limbo.size(), 0)

    def test_returnItem_when_passId(self):
        limbo = Limbo()
        items = get_items(2)
        for idx, item in enumerate(items):
            limbo.add(idx, item)

        eq_(limbo.size(), 2)
        item = limbo.pop(1)
        ok_(item is not None)
        eq_(limbo.size(), 1)
        item = limbo.get(1)
        ok_(item is None)



import unittest
from os import path
from queue import Queue, Full
from unittest.mock import MagicMock

from nose.tools import eq_

from buoy.base.database import DeviceDB
from buoy.base.device.threads.resender import DBToSendThread
from buoy.tests.database import *


class DBToSendThreadTest(unittest.TestCase):
    path_sql = 'tests/support/data'
    db_tablename = "device"
    item_cls = None
    __test__ = False

    def setUp(self):
        db_conf = prepare_db(path.join(self.path_sql, 'setup.sql'))

        self.dev_db = DeviceDB(
            db_config=db_conf,
            db_tablename=self.db_tablename,
            cls_item=self.item_cls
        )

    def test_noPutItemInQueue_when_queueIsFullInLoop(self):
        queue_notice = Queue()
        queue_send_data = Queue()
        queue_send_data.put_nowait = MagicMock(side_effect=Full())
        apply_sql_file(path.join(self.path_sql, 'data_example.sql'))

        thread = DBToSendThread(queue_send_data=queue_send_data, db=self.dev_db, queue_notice=queue_notice)

        thread.activity()

        eq_(queue_send_data.qsize(), 0)

    def test_putItemInQueue_when_queueIsFullInLoop(self):
        queue_notice = Queue()
        queue_send_data = Queue()
        apply_sql_file(path.join(self.path_sql, 'data_5_items_to_send.sql'))

        thread = DBToSendThread(queue_send_data=queue_send_data, db=self.dev_db, queue_notice=queue_notice)

        thread.activity()

        eq_(queue_send_data.qsize(), 5)



import threading
import time
import unittest
from os import EX_OK, EX_OSERR
from unittest.mock import patch

from nose.tools import eq_
from serial import SerialException

import buoy.base.utils.config as load_config
from buoy.tests.database import prepare_db
from buoy.tests.serial import SerialMock


class BaseDeviceTest(unittest.TestCase):
    device_class = None
    DEVICE_NAME = None
    skip_test = False
    __test__ = False
    config_buoy_file = "tests/support/config/buoy.yaml"
    init_db = "tests/support/data/setup.sql"

    def setUp(self):
        if not self.__test__:
            self.skipTest("Skip BaseTest tests, it's a base class")

        self.buoy_config = load_config.load_config(path_config=self.config_buoy_file)
        self.buoy_config['database'] = prepare_db(sql=self.init_db)

    @patch('buoy.base.device.device.Serial', side_effect=SerialMock)
    def test_shouldReturnExitOK_when_stopService(self, mock_serial):

        daemon = self.device_class(name=self.DEVICE_NAME, **self.buoy_config)
        thread = threading.Thread(daemon=True, target=daemon.start)
        thread.start()

        with self.assertRaises(SystemExit) as cm:
            time.sleep(1)
            daemon.stop()

        time.sleep(5)

        eq_(daemon.is_active(), False)
        prefix = '_thread_'
        names = ['reader', 'writer', 'save', 'send', 'reader_from_db']
        for name in names:
            field = prefix + name
            if hasattr(daemon, field):
                thread = getattr(daemon, field)
                is_active = getattr(thread, "is_active")()
                eq_(is_active, False, msg=("Thread %s is active" % (field,)))

        self.assertEqual(cm.exception.code, EX_OK)

    @patch('buoy.base.device.device.Serial', side_effect=SerialException())
    def test_shouldReturnException_when_theDeviceIsNotPresent(self, mock_serial):
        daemon = self.device_class(name=self.DEVICE_NAME, **self.buoy_config)
        with self.assertRaises(SystemExit) as cm:
            time.sleep(1)
            daemon.start()

        self.assertEqual(cm.exception.code, EX_OSERR)


if __name__ == '__main__':
    unittest.main()



import psycopg2
import testing.postgresql

db = None
db_con = None


def prepare_db(sql='setup.sql'):
    """ Module level set-up called once before any tests in this file are executed. Creates a temporary database
    and sets it up """

    global db, db_con

    db = testing.postgresql.Postgresql()
    # Get a map of connection parameters for the database which can be passed
    # to the functions being tested so that they connect to the correct
    # database
    db_conf = db.dsn()
    # Create a connection which can be used by our test functions to set and
    # query the state of the database
    db_con = psycopg2.connect(**db_conf)
    # Commit changes immediately to the database
    db_con.set_isolation_level(psycopg2.extensions.ISOLATION_LEVEL_AUTOCOMMIT)

    # Create the initial database structure (roles, schemas, tables etc.)
    # basically anything that doesn't change
    apply_sql_file(sql)

    return db_conf


def apply_sql_file(sql_path):
    global db_con

    with open(sql_path, 'r') as fh:
        lines_str = fh.read()

    with db_con.cursor() as cur:
        cur.execute(lines_str)


def apply_sql_clause(sql_clause):
    global db_con

    with db_con.cursor(cursor_factory=psycopg2.extras.DictCursor) as cur:
        cur.execute(sql_clause)
        rows = cur.fetchall()

    return rows


def close_db():
    db_con.close()
    db.stop()



# -*- coding: utf-8 -*-

from queue import Queue, Empty
from threading import Thread

from paho.mqtt.client import *

from buoy.base.data.item import ItemQueue, Status
from buoy.base.device.threads.base import BaseThread

logger = logging.getLogger(__name__)


class Limbo(object):
    def __init__(self):
        self.items = dict()

    def add(self, id, item):
        logger.debug("Add item %s with id %s to limbo" % (item, id,))
        self.items[id] = item

    def clear(self):
        self.items.clear()

    def get(self, id):
        if self.exists(id):
            return self.items[id]
        return None

    def pop(self, id):
        item = self.get(id)
        if item:
            del self.items[id]
        logger.debug("Remove item %s with id %s to limbo" % (item, id,))
        return item

    def size(self):
        return len(self.items)

    def exists(self, id):
        return id in self.items


def loop(client):
    client.loop_start()


class MqttThread(BaseThread):
    """
    Clase base encargada de enviar los datos al servidor
    """

    def __init__(self, queue_send_data: Queue, queue_data_sent: Queue, queue_notice: Queue, **kwargs):
        super(MqttThread, self).__init__(queue_notice)

        self.queue_send_data = queue_send_data
        self.queue_data_sent = queue_data_sent
        self.queue_notice = queue_notice
        self.__connected_to_mqtt = False
        self.attemp_connect = False
        self.thread_mqtt = None

        self.client_id = kwargs.pop("client_id", "")
        self.clean_session = kwargs.pop("clean_session", True)
        self.protocol = MQTTv311
        self.transport = kwargs.pop("transport", "tcp")

        self.broker_url = kwargs.pop("broker_url", "iot.eclipse.org")
        self.broker_port = kwargs.pop("broker_port", 1883)
        self.topic_data = kwargs.pop("topic_data", "buoy")
        self.keepalive = kwargs.pop("keepalive", 60)
        self.reconnect_delay = kwargs.pop("reconnect_delay", {"min_delay": 1, "max_delay": 120})

        self.client = MqttClient(client_id=self.client_id, protocol=self.protocol, clean_session=self.clean_session)

        self.limbo = Limbo()

        if "username" in kwargs:
            self.username = kwargs.pop("username", "username")
            self.password = kwargs.pop("password", None)
            self.client.username_pw_set(self.username, self.password)

        self.qos = kwargs.pop("qos", 0)

    def before_activity(self):
        self.connect_to_mqtt()

    def connect_to_mqtt(self):
        logger.info("Try to connect to broker")
        self.client.connect(host=self.broker_url, port=self.broker_port, keepalive=self.keepalive)
        self.client.on_connect = self.on_connect
        self.client.on_disconnect = self.on_disconnect
        self.client.on_publish = self.on_publish

        self.thread_mqtt = Thread(target=loop, args=(self.client,))
        self.thread_mqtt.start()

    def activity(self):
        if self.is_connected_to_mqtt():
            try:
                item = self.queue_send_data.get_nowait()
                self.send(item)
                self.queue_send_data.task_done()
            except Empty:
                logger.debug("No data for sending to broker")
                pass

    def is_connected_to_mqtt(self):
        return self.__connected_to_mqtt

    def send(self, item):
        """

        :param item:
        """
        json = str(item.to_json())
        logger.info("Publish data '%s' to topic '%s'" % (self.topic_data, json))
        try:
            self.limbo.add(item.uuid, item)
            self.client.publish(self.topic_data, json, qos=self.qos, mid=item.uuid)
        except ValueError:
            logger.warning("Can't sent item", exc_info=True)
            self.limbo.pop(item.uuid)
            self.queue_data_sent.put_nowait(ItemQueue(data=item, status=Status.FAILED))
            pass

    def stop(self):
        logger.info("Disconnecting to broker")
        self.client.disconnect()

    def on_publish(self, client, userdata, mid):
        """
        Callback del método publish, si se entra aquí significa que el item fue
        enviado al broker correctamente

        :param client:
        :param userdata:
        :param mid:
        """
        if self.limbo.exists(mid):
            item = self.limbo.pop(mid)
            logger.debug("Update item in db %s", item.uuid)
            self.queue_data_sent.put_nowait(ItemQueue(data=item, status=Status.SENT))
        else:
            logger.warning("Item isn't in limbo")

    def on_connect(self, client, userdata, flags, rc):
        """
        :param client:
        :param userdata:
        :param flags:
        :param rc:
        """
        if rc == 0:
            logger.info("Connected to broker %s with client_id %s", self.broker_url, client)
            self.__connected_to_mqtt = True
            if flags["session present"]:
                logger.info("Connected to broker using existing session")
            else:
                logger.info("Connected to broker using clean session")
        else:
            self.__connected_to_mqtt = False
            if rc == 1:
                logger.error("Connection refused - incorrect protocol version")
            elif rc == 2:
                logger.error("Connection refused - invalid client identifier")
            elif rc == 3:
                logger.error("Connection refused - server unavailable")
            elif rc == 4:
                logger.error("Connection refused - bad username or password")
            elif rc == 5:
                logger.error("Connection refused - not authorised")
            else:
                logger.error("Error connected to broker")

    def on_disconnect(self, client, userdata, rc):
        """

        :param client:
        :param userdata:
        :param rc:
        """
        self.__connected_to_mqtt = False
        self.limbo.clear()
        if rc != 0:
            logger.error("Unexpected disconnection to broker")
        else:
            client.loop_stop()
            super().stop()
        logger.info("Disconnected to broker with result code %s" % str(rc))


try:
    import ssl
except ImportError:
    ssl = None


class MqttClient(Client):
    def __init__(self, client_id="", clean_session=True, userdata=None,
                 protocol=MQTTv311, transport="tcp"):
        super(MqttClient, self).__init__(client_id=client_id, clean_session=clean_session,
                                         userdata=userdata, protocol=protocol, transport=transport)

    def publish(self, topic, payload=None, qos=0, retain=False, mid=None):
        """Publish a message on a topic.

        This causes a message to be sent to the broker and subsequently from
        the broker to any clients subscribing to matching topics.

        topic: The topic that the message should be published on.
        payload: The actual message to send. If not given, or set to None a
        zero length message will be used. Passing an int or float will result
        in the payload being converted to a string representing that number. If
        you wish to send a true int/float, use struct.pack() to create the
        payload you require.
        qos: The quality of service level to use.
        retain: If set to true, the message will be set as the "last known
        good"/retained message for the topic.

        Returns a MQTTMessageInfo class, which can be used to determine whether
        the message has been delivered (using info.is_published()) or to block
        waiting for the message to be delivered (info.wait_for_publish()). The
        message ID and return code of the publish() call can be found at
        info.mid and info.rc.

        For backwards compatibility, the MQTTMessageInfo class is iterable so
        the old construct of (rc, mid) = client.publish(...) is still valid.

        rc is MQTT_ERR_SUCCESS to indicate success or MQTT_ERR_NO_CONN if the
        client is not currently connected.  mid is the message ID for the
        publish request. The mid value can be used to track the publish request
        by checking against the mid argument in the on_publish() callback if it
        is defined.

        A ValueError will be raised if topic is None, has zero length or is
        invalid (contains a wildcard), if qos is not one of 0, 1 or 2, or if
        the length of the payload is greater than 268435455 bytes."""
        if topic is None or len(topic) == 0:
            raise ValueError('Invalid topic.')

        topic = topic.encode('utf-8')

        if self._topic_wildcard_len_check(topic) != MQTT_ERR_SUCCESS:
            raise ValueError('Publish topic cannot contain wildcards.')

        if qos < 0 or qos > 2:
            raise ValueError('Invalid QoS level.')

        if isinstance(payload, unicode):
            local_payload = payload.encode('utf-8')
        elif isinstance(payload, (bytes, bytearray)):
            local_payload = payload
        elif isinstance(payload, (int, float)):
            local_payload = str(payload).encode('ascii')
        elif payload is None:
            local_payload = b''
        else:
            raise TypeError('payload must be a string, bytearray, int, float or None.')

        if len(local_payload) > 268435455:
            raise ValueError('Payload too large.')

        if mid:
            local_mid = mid
        else:
            local_mid = self._mid_generate()

        if qos == 0:
            info = MQTTMessageInfo(local_mid)
            rc = self._send_publish(local_mid, topic, local_payload, qos, retain, False, info)
            info.rc = rc
            return info
        else:
            message = MQTTMessage(local_mid, topic)
            message.timestamp = time_func()
            message.payload = local_payload
            message.qos = qos
            message.retain = retain
            message.dup = False

            with self._out_message_mutex:
                if self._max_queued_messages > 0 and len(self._out_messages) >= self._max_queued_messages:
                    message.info.rc = MQTT_ERR_QUEUE_SIZE
                    return message.info

                if local_mid in self._out_messages:
                    message.info.rc = MQTT_ERR_QUEUE_SIZE
                    return message.info

                self._out_messages[message.mid] = message
                if self._max_inflight_messages == 0 or self._inflight_messages < self._max_inflight_messages:
                    self._inflight_messages += 1
                    if qos == 1:
                        message.state = mqtt_ms_wait_for_puback
                    elif qos == 2:
                        message.state = mqtt_ms_wait_for_pubrec

                    rc = self._send_publish(message.mid, topic, message.payload, message.qos, message.retain,
                                            message.dup)

                    # remove from inflight messages so it will be send after a connection is made
                    if rc is MQTT_ERR_NO_CONN:
                        self._inflight_messages -= 1
                        message.state = mqtt_ms_publish

                    message.info.rc = rc
                    return message.info
                else:
                    message.state = mqtt_ms_queued
                    message.info.rc = MQTT_ERR_SUCCESS
                    return message.info



import unittest
from datetime import datetime, timedelta
from unittest.mock import patch

from nose.tools import eq_

from buoy.base.data.item import BufferItems
from buoy.base.data.utils import convert_to_seconds
from buoy.tests.item import Item, get_items


class TestData(unittest.TestCase):

    def test_shouldReturnNumberSeconds_when_passFewIntervals(self):
        check = [{"in": 1, "out": 1}, {"in": "1", "out": 1}, {"in": "1m", "out": 60}, {"in": "10m", "out": 600}]
        for item in check:
            sec = convert_to_seconds(item["in"])
            eq_(sec, item["out"])

    def test_shouldReturnStatusInOutInterval_when_passFewDates(self):
        buffer = BufferItems(interval="10m")
        date = datetime.strptime("2019-01-01T00:01:59", "%Y-%m-%dT%H:%M:%S")
        check = [{"in": "2019-01-01T00:00:00", "out": False}, {"in": "2019-01-01T00:05:00", "out": True},
                 {"in": "2019-01-01T00:10:10", "out": False}, {"in": "2019-01-01T00:10:00", "out": True}]

        buffer.set_limits(date=date)

        for item in check:
            date = datetime.strptime(item["in"], "%Y-%m-%dT%H:%M:%S")
            eq_(buffer.inside_interval(date=date), item["out"])

    def test_shouldReturnLimitLowerAndHigher_when_setLimit(self):
        buffer = BufferItems(interval="10m")
        date = datetime.strptime("2019-01-01T00:01:59", "%Y-%m-%dT%H:%M:%S")
        limit_lower = datetime.strptime("2019-01-01T00:00:00", "%Y-%m-%dT%H:%M:%S")
        limit_higher = datetime.strptime("2019-01-01T00:10:00", "%Y-%m-%dT%H:%M:%S")

        buffer.set_limits(date)

        eq_((limit_lower, limit_higher,), buffer.limits())

    @unittest.skip
    @patch.object(BufferItems, 'process_buffer')
    def test_shouldThreeProcessCalled_when_passFewDates(self, mock_process_buffer):
        buffer = BufferItems(interval="1m", item_cls=Item)
        dates = []
        date = datetime.strptime("2019-01-01T00:01:01.000", '%Y-%m-%dT%H:%M:%S.%f')
        for i in range(0, 200, 10):
            dates.append(date + timedelta(seconds=i))

        items = get_items(num=len(dates), dates=dates)

        for item in items:
            buffer.append(item)

        eq_(mock_process_buffer.call_count, 3)

    def test_returnCalculatedItem_when_passFewItems(self):
        buffer = BufferItems(interval="1m", item_cls=Item)

        item_expected = Item(date=datetime.strptime("2019-01-01T00:02:00.000", '%Y-%m-%dT%H:%M:%S.%f'), value=20.0)

        item1 = Item(date=datetime.strptime("2019-01-01T00:01:01.000", '%Y-%m-%dT%H:%M:%S.%f'), value=10.0)
        item2 = Item(date=datetime.strptime("2019-01-01T00:01:03.000", '%Y-%m-%dT%H:%M:%S.%f'), value=20.0)
        item3 = Item(date=datetime.strptime("2019-01-01T00:01:50.000", '%Y-%m-%dT%H:%M:%S.%f'), value=30.0)
        item4 = Item(date=datetime.strptime("2019-01-01T00:02:00.001", '%Y-%m-%dT%H:%M:%S.%f'), value=0.0)
        items = [item1, item2, item3, item4]

        items_processed = []
        for i in items:
            item = buffer.append(i)
            if item:
                items_processed.append(item)

        eq_(len(items_processed), 1)
        item_processed = items_processed[0]

        eq_(getattr(item_processed, "date"), getattr(item_expected, "date"))
        eq_(getattr(item_processed, "value"), getattr(item_expected, "value"))


if __name__ == '__main__':
    unittest.main()



from datetime import timedelta

UNITS = {"s": "seconds", "m": "minutes", "h": "hours", "d": "days", "w": "weeks"}


def convert_to_seconds(s):
    if isinstance(s, int):
        # We are dealing with a raw number
        return s

    try:
        seconds = int(s)
        # We are dealing with an integer string
        return seconds
    except ValueError:
        # We are dealing with some other string or type
        pass

    # Expecting a string ending in [m|h|d|s|w]
    count = int(s[:-1])
    unit = UNITS[s[-1]]
    td = timedelta(**{unit: count})
    return td.seconds + 60 * 60 * 24 * td.days


def round_time(dt=None, round_to=1, to='average'):
    seconds = (dt - dt.min).seconds

    if seconds % round_to == 0:
        rounding = (seconds + round_to / 2) // round_to * round_to
    else:
        if to == 'up':
            rounding = (seconds + round_to) // round_to * round_to
        elif to == 'down':
            rounding = seconds // round_to * round_to
        else:
            rounding = (seconds + round_to / 2) // round_to * round_to

    return dt + timedelta(0, rounding - seconds, -dt.microsecond)



# -*- coding: utf-8 -*-

from buoy.client.device.common.item import BaseItem


class WIMDA(BaseItem):
    def __init__(self, **kwargs):
        self.press_inch = kwargs.pop('press_inch', None)
        self.press_mbar = kwargs.pop('press_mbar', None)
        self.air_temp = kwargs.pop('air_temp', None)
        self.water_temp = kwargs.pop('water_temp', None)
        self.rel_humidity = kwargs.pop('rel_humidity', None)
        self.abs_humidity = kwargs.pop('abs_humidity', None)
        self.dew_point = kwargs.pop('dew_point', None)
        self.wind_dir_true = kwargs.pop('wind_dir_true', None)
        self.wind_dir_magnetic = kwargs.pop('wind_dir_magnetic', None)
        self.wind_knots = kwargs.pop('wind_knots', None)
        self.wind_meters = kwargs.pop('wind_meters', None)
        super(WIMDA, self).__init__(**kwargs)

    @staticmethod
    def from_nmea(in_datetime, wimda):
        if hasattr(wimda, "b_pressure_bar"):
            wimda.b_pressure_bar *= 1000

        return WIMDA(
            date=in_datetime,
            press_inch=wimda.b_pressure_inch,
            press_mbar=wimda.b_pressure_bar,
            air_temp=wimda.air_temp,
            water_temp=wimda.water_temp,
            rel_humidity=wimda.rel_humidity,
            abs_humidity=wimda.abs_humidity,
            dew_point=wimda.dew_point,
            wind_dir_true=wimda.direction_true,
            wind_dir_magnetic=wimda.direction_magnetic,
            wind_knots=wimda.wind_speed_knots,
            wind_meters=wimda.wind_speed_meters)

    @property
    def press_inch(self):
        """
        :return: Barometric pressure, inches of mercury
        :rtype: Decimal
        """
        return self._press_inch

    @press_inch.setter
    def press_inch(self, value):
        self._press_inch = self._convert_string_to_decimal(value)

    @property
    def press_mbar(self):
        """
        :return: Barometric pressure, bars
        :rtype: Decimal
        """
        return self._press_mbar

    @press_mbar.setter
    def press_mbar(self, value):
        self._press_mbar = self._convert_string_to_decimal(value)

    @property
    def air_temp(self):
        """
        :return: Barometric pressure, bars
        :rtype: Decimal
        """
        return self._air_temp

    @air_temp.setter
    def air_temp(self, value):
        self._air_temp = self._convert_string_to_decimal(value)

    @property
    def water_temp(self):
        """
        :return: Water temperature, degrees Celsius
        :rtype: Decimal
        """
        return self._water_temp

    @water_temp.setter
    def water_temp(self, value):
        self._water_temp = self._convert_string_to_decimal(value)

    @property
    def rel_humidity(self):
        """
        :return: Relative humidity, percent
        :rtype: Decimal
        """
        return self._rel_humidity

    @rel_humidity.setter
    def rel_humidity(self, value):
        self._rel_humidity = self._convert_string_to_decimal(value)

    @property
    def abs_humidity(self):
        """
        :return: Absolute humidity, percent
        :rtype: Decimal
        """
        return self._abs_humidity

    @abs_humidity.setter
    def abs_humidity(self, value):
        self._abs_humidity = self._convert_string_to_decimal(value)

    @property
    def dew_point(self):
        """
        :return: Dew point, degrees C
        :rtype: Decimal
        """
        return self._dew_point

    @dew_point.setter
    def dew_point(self, value):
        self._dew_point = self._convert_string_to_decimal(value)

    @property
    def wind_dir_true(self):
        """
        :return: Wind direction true
        :rtype: Decimal
        """
        return self._wind_dir_true

    @wind_dir_true.setter
    def wind_dir_true(self, value):
        self._wind_dir_true = self._convert_string_to_decimal(value)

    @property
    def wind_dir_magnetic(self):
        """
        :return: Wind direction magnetic
        :rtype: Decimal
        """
        return self._wind_dir_magnetic

    @wind_dir_magnetic.setter
    def wind_dir_magnetic(self, value):
        self._wind_dir_magnetic = self._convert_string_to_decimal(value)

    @property
    def wind_knots(self):
        """
        :return: Wind speed knots
        :rtype: Decimal
        """
        return self._wind_knots

    @wind_knots.setter
    def wind_knots(self, value):
        self._wind_knots = self._convert_string_to_decimal(value)

    @property
    def wind_meters(self):
        """
        :return: Wind speed meters/second
        :rtype: Decimal
        """
        return self._wind_meters

    @wind_meters.setter
    def wind_meters(self, value):
        self._wind_meters = self._convert_string_to_decimal(value)

    def __str__(self):
        return ("Uuid: {uuid}\n"
                "Date: {date}\n"
                "Barometric pressure: {press_inch} in - {press_mbar} mbar\n"
                "Air temperature: {air_temp} ºC\n"
                "Water temperature: {water_temp} ºC\n"
                "Relative humidity: {rel_humidity} %\n"
                "Absolute humidity: {abs_humidity} %\n"
                "Dew point: {dew_point} C\n"
                "Wind direction true: {wind_dir_true} º\n"
                "Wind direction magnetic: {wind_dir_magnetic} º\n"
                "Wind speed: {wind_knots} knots - {wind_meters} m/s").format(**dict(self))



import unittest
from queue import Queue
from unittest.mock import patch, call

from nose.tools import eq_

from buoy.base.data.item import ItemQueue, Status
from buoy.base.device.threads.save import SaveThread


def get_item():
    pass


def get_items(num=2):
    items = []
    for i in range(0, num):
        items.append(get_item())
    return items


class ItemSaveThreadTest(unittest.TestCase):
    __test__ = False

    @patch.object(SaveThread, 'save')
    @patch.object(SaveThread, 'set_sent')
    @patch.object(SaveThread, 'set_failed')
    @patch.object(SaveThread, 'is_active', side_effect=[True, True, True, False])
    def test_callActionDb_when_insertItemsWithVariousStatusInQueueData(self, mock_is_active, mock_set_failed,
                                                                       mock_set_sent, mock_save):
        queue_data = Queue()
        queue_notice = Queue()

        items = [ItemQueue(data=get_item()), ItemQueue(data=get_item(), status=Status.SENT),
                 ItemQueue(data=get_item(), status=Status.FAILED)]

        for item in items:
            queue_data.put_nowait(item)

        thread = SaveThread(queue_save_data=queue_data, db=None, queue_notice=queue_notice)
        thread.run()

        eq_(mock_is_active.call_count, 4)
        eq_(mock_save.call_count, 1)
        eq_(mock_save.call_args, call(items[0].data))
        eq_(mock_set_sent.call_args, call(items[1].data))
        eq_(mock_set_failed.call_args, call(items[2].data))



import os
import shutil
import signal
import threading
import time
import unittest
from os.path import exists, join
from unittest.mock import patch

from nose.tools import eq_, ok_

from buoy.base.service.daemon import Daemon


class DaemonTest(Daemon):
    def __init__(self, daemon_name, daemon_config):
        super(DaemonTest, self).__init__(daemon_name=daemon_name, daemon_config=daemon_config)
        self.nun_attempts = 0
        self.max_attempts = 4

    def run(self):
        while self.is_active():
            time.sleep(0.01)


class TestDaemon(unittest.TestCase):
    def setUp(self):
        self.path_pidfile = './tests/support/pids/'
        self.name = 'DaemonTest'
        self.config = {
            'path_pidfile': self.path_pidfile
        }

        # Limpia el directorio donde se almacenan los PID
        if exists(self.path_pidfile):
            shutil.rmtree(self.path_pidfile)

    def run_daemon_in_thread(self):
        self.daemon = DaemonTest(daemon_name=self.name, daemon_config=self.config)
        t = threading.Thread(target=self.daemon.start)
        t.start()

    def test_should_createPathPID_when_noExits(self):
        self.daemon = Daemon(daemon_name=self.name, daemon_config=self.config)

        eq_(self.daemon.is_active(), False)
        eq_(self.daemon.daemon_name, self.name)
        ok_(exists(self.config['path_pidfile']))

    @patch.object(Daemon, '_before_start', return_value=None)
    @patch.object(Daemon, 'run', return_value=None)
    @patch.object(Daemon, '_stop', return_value=None)
    def test_should_lifecycle_when_callStart(self, mock_stop, mock_run, mock_before_start):
        self.daemon = Daemon(daemon_name=self.name, daemon_config=self.config)
        self.daemon.start()

        eq_(self.daemon.is_active(), False)
        eq_(mock_before_start.call_count, 1)
        eq_(mock_run.call_count, 1)
        eq_(mock_stop.call_count, 1)

    def test_should_stopDaemonAndCleanPIDFile_when_callStopMethod(self):
        self.run_daemon_in_thread()

        time.sleep(0.2)
        eq_(self.daemon.is_active(), True)
        ok_(exists(join(self.path_pidfile, self.name + ".pid")))

        time.sleep(0.2)
        with self.assertRaises(SystemExit) as cm:
            self.daemon.stop()

        self.assertEqual(cm.exception.code, 0)

        eq_(self.daemon.is_active(), False)
        ok_(not exists(join(self.path_pidfile, self.name + ".pid")))

    # TODO Arreglar tests
    @unittest.skip
    def test_should_stopDaemonAndCleanPIDFile_when_sendSignalSIGINT(self):
        self.run_daemon_in_thread()
        eq_(self.daemon.is_active(), True)

        time.sleep(0.2)
        os.kill(self.daemon.pid, signal.SIGINT)

        eq_(self.daemon.is_active(), False)

    @unittest.skip
    def test_should_stopDaemonAndCleanPIDFile_when_sendSignalSIGTERM(self):
        self.run_daemon_in_thread()
        eq_(self.daemon.is_active(), True)

        time.sleep(0.2)
        os.kill(self.daemon.pid, signal.SIGTERM)

        eq_(self.daemon.is_active(), False)

    @patch.object(Daemon, 'is_active', return_value=True)
    def test_should_exitWithCode0_when_callStop(self, mock_is_active):
        self.daemon = Daemon(daemon_name=self.name, daemon_config=self.config)

        with self.assertRaises(SystemExit) as cm:
            self.daemon.stop()

        self.assertEqual(cm.exception.code, os.EX_OK)

    @patch.object(Daemon, 'is_active', return_value=True)
    def test_should_exitWithCode1_when_callError(self, mock_is_active):
        self.daemon = Daemon(daemon_name=self.name, daemon_config=self.config)

        with self.assertRaises(SystemExit) as cm:
            self.daemon.error()

        self.assertEqual(cm.exception.code, os.EX_OSERR)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

import json
import logging
from datetime import datetime, timezone, timedelta
from decimal import *
from enum import Enum
from uuid import uuid4, UUID
from buoy.base.data.utils import convert_to_seconds, round_time

from dateutil import parser

logger = logging.getLogger(__name__)


class BaseItem(object):
    def __init__(self, **kwargs):
        self.uuid = kwargs.pop('uuid', uuid4())
        self.date = kwargs.pop('date', datetime.now(tz=timezone.utc))

    @property
    def uuid(self):
        """
        :return: Identifier
        :rtype: Integer
        """
        return self._uuid

    @uuid.setter
    def uuid(self, value):
        self._uuid = value

    @property
    def date(self):
        """
        :return: Datetime
        :rtype: Datetime
        """
        return self._date

    @date.setter
    def date(self, value):
        if type(value) is int:
            value = datetime.fromtimestamp(value / 1000.0)
        elif type(value) is str:
            value = parser.parse(value)

        self._date = value

    @staticmethod
    def _convert_string_to_decimal(value):
        val = None
        if value is not None:
            try:
                val = Decimal(value)
            except InvalidOperation:
                logger.error("Convert string to decimal", value)

        return val

    def to_json(self):
        item = json.dumps(self, cls=DataEncoder, sort_keys=True, separators=(',', ':'))
        return item

    def __iter__(self):
        for name in dir(self):
            yield name, getattr(self, name)

    def __dir__(self):
        list_props = []
        for name in vars(self):
            list_props.append(name[1:])

        return list_props

    def __eq__(self, other):
        if isinstance(other, self.__class__):
            a = dict(other)
            b = dict(self)
            return a == b
        return False

    def __lt__(self, other):
        if isinstance(other, self.__class__):
            return self.date < other.date

    def __str__(self):
        return ("Uuid: {uuid}\n"
                "Date: {date}\n").format(**dict(self))

    def __copy__(self):
        cls = self.__class__
        result = cls.__new__(cls)
        result.__dict__.update(self.__dict__)
        return result


class DataEncoder(json.JSONEncoder):
    def default(self, o):
        serial = {}
        for name in dir(o):
            value = getattr(o, name)
            datatype = type(value)
            if datatype is datetime:
                serial[name] = value.isoformat(timespec='milliseconds')
            elif datatype is Decimal:
                serial[name] = round(float(value), 3)
            elif datatype is int:
                serial[name] = value
            elif datatype is str:
                serial[name] = value
            elif datatype is UUID:
                serial[name] = str(value)
            elif isinstance(value, BaseItem):
                serial[name] = self.default(value)
            elif value:
                try:
                    serial[name] = json.JSONEncoder.default(self, value)
                except TypeError as e:
                    logger.error("No serialize property %s with value %s" % (name, value,))

        return serial


class Status(Enum):
    NEW = 0
    SENT = 1
    FAILED = 2


class ItemQueue(object):
    def __init__(self, data: BaseItem, **kwargs):
        self.status = kwargs.pop("status", Status.NEW)
        self.data = data


class BufferItems(object):
    def __init__(self, **kwargs):
        self.__buffer = []
        self.interval = kwargs.pop("interval", None)
        if self.interval:
            self.interval = convert_to_seconds(self.interval)

        self.__limit_higher = None
        self.__limit_lower = None
        self.__item_cls = kwargs.pop("item_cls", BaseItem)
        self.__fields = ["value"]

    def append(self, other: BaseItem):
        item = None

        if other is None:
            return item

        if self.interval is None:
            item = other
        elif self.inside_interval(other.date):
            if len(self.__buffer) == 0:
                self.set_limits(other.date)
            self.__buffer.append(other)
        else:
            item = self.process_buffer()
            self.clear()
            self.append(other)

        return item

    def set_limits(self, date):
        self.__limit_lower = round_time(dt=date, round_to=self.interval, to="down")
        self.__limit_higher = self.__limit_lower + timedelta(seconds=self.interval)

    def clear(self):
        self.__buffer.clear()
        self.__limit_lower = None
        self.__limit_higher = None

    def limits(self):
        return self.__limit_lower, self.__limit_higher

    def inside_interval(self, date):
        return self.__limit_higher is None and self.__limit_lower is None or \
               (self.__limit_lower < date <= self.__limit_higher)

    def process_buffer(self):
        item_attr = {
            "date": self.__limit_higher
        }
        for key in self.__fields:
            attr = [getattr(o, key, None) for o in self.__buffer]
            item_attr[key] = sum(attr) / len(attr)

        return self.__item_cls(**item_attr)



# -*- coding: utf-8 -*-

from datetime import datetime, timezone
from enum import IntEnum


class ExceptionLevel(IntEnum):
    LOW = 10
    NORMAL = 5
    HIGHT = 3
    CRITICAL = 1


class DeviceBaseException(Exception):
    def __init__(self, message, exception: Exception, level=ExceptionLevel.LOW, **kwargs):
        self.proccess = kwargs.pop('proccess', None)
        self.message = message
        self.level = level
        self.datetime = datetime.now(tz=timezone.utc)
        self.exception = exception


class ConnectionException(DeviceBaseException):
    def __init__(self, message, exception: Exception, level=ExceptionLevel.CRITICAL, **kwargs):
        super().__init__(message=message, exception=exception, level=level, **kwargs)


class LostConnectionException(DeviceBaseException):
    def __init__(self, exception: Exception, message="Lost your connection to the device",
                 level=ExceptionLevel.CRITICAL, **kwargs):
        super().__init__(message=message, exception=exception, level=level, **kwargs)


class DeviceNoDetectedException(DeviceBaseException):
    def __init__(self, exception: Exception, message="Device no detected", level=ExceptionLevel.CRITICAL, **kwargs):
        super().__init__(message=message, exception=exception, level=level, **kwargs)


class ProcessDataExecption(DeviceBaseException):
    def __init__(self, exception: Exception, message="Error processed data",
                 level=ExceptionLevel.CRITICAL, **kwargs):
        super().__init__(message=message, exception=exception, level=level, **kwargs)



# -*- coding: utf-8 -*-

import logging
from queue import Queue, Empty

from serial import Serial, SerialException

from buoy.base.device.exceptions import LostConnectionException
from buoy.base.device.threads.base import DeviceBaseThread

logger = logging.getLogger(__name__)


class DeviceWriter(DeviceBaseThread):
    """ Clase encargada de enviar datos al dispositivo """

    def __init__(self, device: Serial, queue_write_data: Queue, queue_notice: Queue):
        super(DeviceWriter, self).__init__(device, queue_notice)
        self.queue_write_data = queue_write_data

    def activity(self):
        try:
            data = self.queue_write_data.get(timeout=self.timeout_wait)
            self.device.write(data.encode())
            logger.info("Write data in device - " + data)
            self.queue_write_data.task_done()
        except SerialException as ex:
            logger.error("Device disconnected")
            self.error(LostConnectionException(exception=ex))
        except Empty:
            pass
